@@ -1,6 +1,6 @@
 """
 .. module:: create
-:synopsis: Import a Word `docx` document, define its metadata, cover and rights, and publish it as an EPUB3.
+  :synopsis: Import a Word `docx` document, define its metadata, cover and rights, and publish it as an EPUB3.
 
 .. moduleauthor:: Gavin Chait <github.com/turukawa>
 
@@ -16,7 +16,12 @@ graphics.
 Workflow
 --------
 
-The publication process runs as follows:
+There are two main publication approaches, stateless and non-stateless. A non-stateless approach assumes you may be 
+starting each step discretely (perhaps via a set of one-time network calls). The second maintains state, so you can
+complete the process in one step.
+
+
+The *stateless* publication process runs as follows:
 
 * Set the working directory on creation,
 * Define and validate the metadata required for the creative work,
@@ -28,10 +33,22 @@ The publication process runs as follows:
 * Build the creative work,
 * Validate the work is EPUB3 standards compliant.
 
-The objective of the workflow is to support what may be a stateless process i.e. the individual steps first bring all
+The objective of this workflow is to support what may be a stateless process i.e. the individual steps first bring all
 the data required to produce the creative work into a project directory, and then produces it. State does not need
 to be maintained between steps.
 
+The *non-stateless* process runs as follows:
+
+* Define and validate the metadata required for the creative work,
+* Supply the `docx` file as a base64 string,
+* Copy the cover image as a base64 string,
+* Add in an optional dedication,
+* Build the creative work,
+* Validate the work is EPUB3 standards compliant.
+
+The objective in a non-stateless workflow is to minimise disruption, and store the minimum amount of information. Only
+the epub itself will be saved, and then only because Pandoc does not support a memory-only epub build.
+
 Build your work
 ---------------
 
@@ -43,7 +60,15 @@ Import **Chapisha** and create a work:
 
     work = CreateWork(directory)
 
-Where `directory` is the complete path to where you would like the EPUB created.
+Where `directory` is the complete path to where you would like the EPUB created. If you want a stateless workflow, 
+set the `stateless` boolean to `True`. If you already have the `metadata` (perhaps via a web form), you can skip 
+several steps and pick up again for setting the files and images.
+
+.. code-block:: python
+
+    from chapisha.create import CreateWork
+
+    work = CreateWork(directory, metadata=metadata, stateless=True)
 
 Set metadata
 ^^^^^^^^^^^^
@@ -122,7 +147,7 @@ to a printed work.
 
     work.set_document(source)
 
-Where `source` is the complete path to the source `docx` file.
+Where `source` is any of the complete path to the source `docx` file, a `bytes` file import, or a `base64` string.
 
 Set cover
 ^^^^^^^^^
@@ -150,7 +175,7 @@ Example code:
 
     work.set_cover(source, contributor=CONTRIBUTOR)
 
-Where `source` is the complete path to the image file.
+Where `source` is the complete path to the image file, a `bytes` file import, or a `base64` string.
 
 Add contributors
 ^^^^^^^^^^^^^^^^
@@ -267,10 +292,12 @@ Output will be `True` or `False`.
 import pypandoc
 from bs4 import BeautifulSoup
 from epubcheck import EpubCheck
-from typing import Optional
+from typing import Optional, Literal, List
 from urllib.parse import urlparse
 from pathlib import Path
 import os
+import re
+import base64
 import filetype
 
 from ..models.metadata import WorkMetadata, Contributor
@@ -289,9 +316,26 @@ class CreateWork:
     or replacing with specified defaults.
     """
 
-    def __init__(self, directory: Optional[str] = None, metadata: Optional[WorkMetadata] = None):
+    def __init__(self, 
+                 directory: Optional[str] = None, 
+                 metadata: Optional[WorkMetadata] = None, 
+                 stateless: bool = False):
+        """
+        Initialise the CreateWork class.
+
+        Parameters
+        ----------
+        directory: str
+            A directory path where you would like to save your work.
+        metadata: WorkMetadata
+            A model defined by a dictionary of terms.
+        stateless: bool
+            Whether your workflow is stateless (default False).
+        """
+        self.stateless = stateless
         self.directory = Path(directory)
-        _c.check_path(self.directory)
+        if self.stateless:
+            _c.check_path(self.directory)
         # Load metadata settings, if exists
         try:
             _c.check_source(self.directory / _c.DEFAULT_METADATA_SETTINGS)
@@ -301,8 +345,15 @@ class CreateWork:
             self.metadata = None
             self.work_name = None
         # Construct the metadata, if it is provided
-        if metadata: self.set_metadata(**metadata)
+        if metadata:
+            if isinstance(metadata, WorkMetadata):
+                metadata = metadata.dict()
+            self.set_metadata(metadata)
         self.source_path = _c.get_helper_path() / "data" 
+        # Set default cover and work bytes
+        self.work = None
+        self.cover = None
+        self.dedication = None
 
     ############################################################################
     # GATHER WORKING DATA
@@ -348,19 +399,24 @@ class CreateWork:
         if not self.work_name:
             self.work_name = work_name
             self.directory = self.directory / work_name
-        # Save the metadata to the working folder
-        _c.check_path(self.directory)
-        _c.save_json(self.metadata.dict(by_alias=True), self.directory / _c.DEFAULT_METADATA_SETTINGS, overwrite=True)
+        # If stateless, save the metadata to the working folder
+        if self.stateless:
+            _c.check_path(self.directory)
+            _c.save_json(self.metadata.dict(by_alias=True), self.directory / _c.DEFAULT_METADATA_SETTINGS, overwrite=True)
         return True
 
-    def _get_validated_bytes(self, source: [Path, bytes]) -> bytes:
+    def _get_validated_bytes(self, 
+                             source: [Path, bytes], 
+                             base_type: Optional[List[Literal["cover", "work"]]] = None) -> bytes:
         """
         Validate a source file, and return a bytes version.
 
         Parameters
         ----------
-        source: Path or bytes
-            Filename to open, or bytes from an opened file
+        source: Path, bytes or base64 string
+            Filename to open, base64 string, or bytes from an opened file
+        base_type: Optional, str
+            Must be one of "cover" or "work" for interpreting base64 mime type
 
         Raises
         ------
@@ -382,19 +438,26 @@ class CreateWork:
             except FileNotFoundError:
                 e = F"`{source}` is not a valid file source."
                 raise FileNotFoundError(e)
+        if isinstance(source, str) and base_type:
+            # Base64 string, remove any provided mime type
+            source_type = re.search(_c.DEFAULT_BASE64_TYPES[base_type], source)
+            if source_type:
+                source = source.replace(source_type.group(0), "")
+            source = base64.b64decode(source)
         if not isinstance(source, bytes):
             e = F"File is not valid."
             raise FileNotFoundError(e)
         return source
 
-    def set_document(self, source: [Path, bytes]):
+    def set_document(self, source: [Path, bytes, str]):
         """
-        Import source `docx` document and save to the working directory..
+        Import source `docx` document and, if stateless, save to the working directory. If you're finding errors in 
+        the build step, it could be you need to convert your base64 string to "utf-8" (`source.decode("utf-8")`).
 
         Parameters
         ----------
-        source: Path or bytes
-            Filename to open, or bytes from an opened file
+        source: Path, bytes, or str
+            Filename to open, bytes from an opened file, or a base64 string
 
         Raises
         ------
@@ -404,15 +467,20 @@ class CreateWork:
         if not self.work_name or not self.metadata:
             e = "`set_metadata` before setting source document."
             raise PermissionError(e)
-        source = self._get_validated_bytes(source)
-        with open(self.directory / F"{self.work_name}.docx", "wb") as w:
-            w.write(source)
+        source = self._get_validated_bytes(source, base_type = "work")
+        if self.stateless:
+            with open(self.directory / F"{self.work_name}.docx", "wb") as w:
+                w.write(source)
+        else:
+            self.work = source
 
     def set_cover(self, 
                   source: [Path, bytes],
                   contributor: Optional[Contributor] = None):
         """
-        Import cover image and save to the working directory, along with any rights and contributor information.
+        Import cover image and, if stateless, save to the working directory, along with any rights and contributor 
+        information. If you're finding errors in the build step, it could be you need to convert your base64 string to 
+        "utf-8" (`source.decode("utf-8")`).
 
         Parameters
         ----------
@@ -429,15 +497,22 @@ class CreateWork:
         if not self.work_name or not self.metadata:
             e = "`set_metadata` before setting cover."
             raise PermissionError(e)
-        source = self._get_validated_bytes(source)
-        kind = filetype.guess(source).extension
-        with open(self.directory / F"cover.{kind}", "wb") as w:
-            w.write(source)
+        # Cover contributor
         if contributor: 
             if self.metadata.contributor is None:
                 self.metadata.contributor = []
             self.metadata.contributor.append(Contributor(**contributor))
-        _c.save_json(self.metadata.dict(by_alias=True), self.directory / _c.DEFAULT_METADATA_SETTINGS, overwrite=True)
+        # Cover image
+        source = self._get_validated_bytes(source, base_type = "cover")
+        if self.stateless:
+            kind = filetype.guess(source).extension
+            with open(self.directory / F"cover.{kind}", "wb") as w:
+                w.write(source)
+            _c.save_json(self.metadata.dict(by_alias=True), 
+                                            self.directory / _c.DEFAULT_METADATA_SETTINGS, 
+                                            overwrite=True)
+        else:
+            self.cover = source
 
     def add_contributor(self, contributor: Contributor):
         """
@@ -482,8 +557,10 @@ class CreateWork:
         if not self.work_name or not self.metadata:
             e = "`set_metadata` before setting dedication."
             raise PermissionError(e)
-        with open(self.directory / F"dedication.xhtml", "w") as w:
-            w.write(pages.create_dedication_xhtml(dedication))
+        self.dedication = pages.create_dedication_xhtml(dedication)
+        if self.stateless:
+            with open(self.directory / F"dedication.xhtml", "w") as w:
+                w.write(self.dedication)
 
     def set_rights(self, rights: [str, list[str]]):
         """
@@ -540,10 +617,17 @@ class CreateWork:
         epub_path = self.directory.parent / F"{self.work_name}.epub"
         # Generate the initial creative content using Pandoc
         # pypandoc can't handle PosixPaths ...
-        pypandoc.convert_file(str(self.directory / F"{self.work_name}.docx"), 
-                              format="docx",
-                              to="epub3",
-                              outputfile=str(epub_path))
+        if self.stateless:
+            pypandoc.convert_file(str(self.directory / F"{self.work_name}.docx"), 
+                                  format="docx",
+                                  to="epub3",
+                                  outputfile=str(epub_path))
+        else:
+            # Maybe one day Pandoc can return an epub object and we won't save the interim file
+            pypandoc.convert_text(self.work, 
+                                  format="docx",
+                                  to="epub3",
+                                  outputfile=str(epub_path))
         # Generate the epub version
         with UpdateZipFile(epub_path, "a") as w:
             # REMOVES
@@ -563,15 +647,21 @@ class CreateWork:
             # ADD colophon.xhtml
             w.writestr("EPUB/text/colophon.xhtml", pages.create_colophon_xhtml(self.metadata))
             # ADD cover.img
-            for custom_image in ["cover"]:
-                for image_path in [self.directory / F"{custom_image}.{t}" for t in ["jpg", "jpeg", "png", "gif", "svg"]]:
+            if self.stateless:
+                for image_path in [self.directory / F"cover.{t}" for t in ["jpg", "jpeg", "png", "gif", "svg"]]:
                     if image_path.exists():
                         w.write(image_path, F"EPUB/images/{image_path.name}")
+            elif self.cover:
+                t = filetype.guess(self.cover).extension
+                w.writestr(F"EPUB/images/cover.{t}", self.cover)
             # GET DEDICATION and CHAPTERS
             spine = []
             # check if the path to dedication exists, if it does, add it to the work and spine
-            if (self.directory / "dedication.xhtml").exists():
-                w.write(self.directory / "dedication.xhtml", "EPUB/text/dedication.xhtml")
+            if (self.directory / "dedication.xhtml").exists() or self.dedication:
+                if self.dedication: 
+                    w.writestr("EPUB/text/dedication.xhtml", self.dedication)
+                else:
+                    w.write(self.directory / "dedication.xhtml", "EPUB/text/dedication.xhtml")
                 spine = [Matter(partition="frontmatter", content="dedication", title="Dedication")]
             CHAPTERS = [f for f in w.namelist() if f.startswith("EPUB/text/ch")]
             CHAPTERS.sort()
