@@ -97,8 +97,8 @@ def create_titlepage_xhtml(metadata: WorkMetadata) -> str:
         xml_txt = xml_txt.replace('<text class="contributor-descriptor" x="700" y="480">Generated by Chapisha</text>',
                                   F'<text class="contributor-descriptor" x="700" y="{y}">Generated by Chapisha</text>')
         y += SECTION_SPACER - TITLEPAGE_ROW_MARGIN * 2
-        xml_txt = xml_txt.replace('<image x="650" y="540" width="100" height="100" ns:href="../images/logo.svg" />',
-                                  F'<image x="650" y="{y}" width="140" height="140" ns:href="../images/logo.svg" />')
+        xml_txt = xml_txt.replace('<image x="650" y="540" width="100" height="100" xlink:href="../images/logo.svg" />',
+                                  F'<image x="650" y="{y}" width="140" height="140" xlink:href="../images/logo.svg" />')
         y += 150
         xml_txt = xml_txt.replace(F'viewBox="0 0 1400 700"',
                                 F'viewBox="0 0 1400 {y}"')
@@ -147,7 +147,9 @@ def create_content_opf(metadata: WorkMetadata, image_manifest: list[str], spine:
                                   F'<dc:title id="title">{metadata.title}</dc:title>\n\t\t<meta property="file-as" refines="#title">{metadata.title}</meta>')
         # dc:subject
         subject_xml = ""
-        if len(metadata.subject):
+        if metadata.subject and len(metadata.subject):
+            if isinstance(metadata.subject, str):
+                metadata.subject = [metadata.subject]
             for i, subject in enumerate(metadata.subject):
                 subject_xml += F'\t\t<dc:subject id="subject-{i+1}">{subject}</dc:subject>\n'
         opf_xml = opf_xml.replace('\t\t<dc:subject id="subject-1"></dc:subject>\n', 
@@ -161,7 +163,8 @@ def create_content_opf(metadata: WorkMetadata, image_manifest: list[str], spine:
         # meta long-description
         long_description_xml = ""
         if metadata.long_description:
-            long_description_xml = F'<meta id="long-description" property="se:long-description" refines="#description">{metadata.long_description}</meta>'
+            long_description = "\n\n".join(formats.get_text_paragraphs(metadata.long_description))
+            long_description_xml = F'<meta id="long-description" property="se:long-description" refines="#description">{long_description}</meta>'
         opf_xml = opf_xml.replace('<meta id="long-description" property="se:long-description" refines="#description"></meta>', 
                                   long_description_xml)
         # dc:language
@@ -181,7 +184,7 @@ def create_content_opf(metadata: WorkMetadata, image_manifest: list[str], spine:
                                   creator_xml)
         # dc:contributor
         contributor_xml = ""
-        if len(metadata.contributor):
+        if metadata.contributor and len(metadata.contributor):
             for i, contributor in enumerate(metadata.contributor):
                 contributor_xml += F'\t\t<dc:contributor id="{contributor.role}-{i+1}">{contributor.name}</dc:contributor>\n'
         opf_xml = opf_xml.replace('\t\t<dc:contributor id="artist"></dc:contributor>\n', 
@@ -329,7 +332,7 @@ def create_colophon_xhtml(metadata: WorkMetadata) -> str:
         xml_txt = xml_txt.replace('<p><a href="AUTHOR_URL">AUTHOR_URL</a><br/></p>', xml_url)
         # Set publication long-rights
         xml_rights = ""
-        for p in metadata.long_rights:
+        for p in formats.get_text_paragraphs(metadata.long_rights):
             xml_rights += F"\t\t\t<p>{p}</p>\n"
         xml_txt = xml_txt.replace("\t\t\t<p>PUBLICATION_RIGHTS</p>\n", xml_rights)
         # Set publisher and publisher url
@@ -341,7 +344,9 @@ def create_colophon_xhtml(metadata: WorkMetadata) -> str:
         xml_txt = xml_txt.replace("<p><br/>Published by PUBLISHER.</p>", xml_pub)
         # Set contributors
         xml_ctrb = ""
-        for ctrb in metadata.contributor:
+        for ctrb in [] if not metadata.contributor else metadata.contributor:
+            # https://stackoverflow.com/a/38821619/295606
+            if not ctrb.name: continue
             # If ctrb.year is None, then use work year
             if not ctrb.year:
                 ctrb.year = metadata.isodate.year
@@ -362,12 +367,12 @@ def create_dedication_xhtml(dedication: [str, list[str]]) -> str:
     -------
     str: xhtml response for `dedication.xhtml` as a str.
     """
-    if isinstance(dedication, str):
-        dedication = [dedication]
+    if isinstance(dedication, list):
+        dedication = "\n".join(dedication)
     with open(DATA_PATH / "xhtml" / DEFAULT_DEDICATION_XHTML, "r+", encoding="utf-8") as xml_file:
         xml_txt = xml_file.read()
         ddctn_xml = ""
-        for p in dedication:
+        for p in formats.get_text_paragraphs(dedication):
             ddctn_xml += F"\t\t\t<p>{p}</p>\n"
         xml_txt = xml_txt.replace("\t\t\tDEDICATION\n", ddctn_xml)
     return xml_txt
\ No newline at end of file
